package parametriche;

import java.util.Collection;

/**
* Classe che realizza una collezione appoggiandosi ad un'array.
* Non Permette elementi duplicati. Permette null.
*/
public abstract class ArrayCollection<E>
implements Collection<E>
{
	E[] storage = null;
	int count;

	public ArrayCollection(int dimMax)
	{
		storage = (E[])(new Object[dimMax]);
		count = 0;
	}

	public ArrayCollection()
	{
		this(10);
	}

	/**
	* Metodo di inserimento alla fine della collezione.
	* Non permette elementi duplicati. Accetta null.
	* Ritorna true se la collezione e' modificata dall'operazione
	* corrente.
	*/
	public boolean add(E elem)
	{
		//dummy implementation secondo la convenzione delle optional operations
		//throw new UnsupportedOperationException("torna domani");
		if(elem == null)
		{
			for(int i = 0;i<count;i++)
			{
				if(storage[i] == null)
				{
					return false;
				}
			}
		}
		else
		{
			for(int i = 0;i<count;i++)
			{
				if(elem.equals(storage[i]))
				{
					return false;
				}
			}
		}
		try
		{
			storage[count++] = elem;
		}
		catch(ArrayIndexOutOfBoundsException aioobe)
		{
			IllegalStateException ise = new IllegalStateException("Collection full");
			ise.initCause(aioobe);
			throw ise;
		}
		return true;
	}

	public boolean addAll(Collection<? extends E> c)
	{
		boolean retval = false;
		Iterator<? extends E> iter = c.iterator();
		while(iter.hasNext())
		{
			retval = retval || add(iter.next());
			/* Versione passo dopo passo
			Object tmp = iter.next();
			boolean tmpB = add(tmp);
			retval = retval || tmpB;
			*/
		}
		return retval;
	}

	public void clear()
	{
		for(int i = 0;i<count;i++)
		{
			storage[i] = null;		
		}
		count = 0;
	}

	public boolean equals(Object o)
	{
		if((o instanceof Set) || (o instanceof List))
		{
			//da contratto nella documentazione: devo ritornare false
			return false;
		}

		/* Per un debug, ha senso voler ottenere l'eccezione
		if(!(o instanceof Collection))
		{
			return false;
		}
		*/

		Collection c = (Collection)o;
		//Vediamo se hanno la stessa lunghezza;
		if(c.size() != size())
		{
			return false;
		}

		//hanno la stessa lunghezza, mi tocca proprio confrontare
		// gli elementi a coppie.
		Iterator iter = c.iterator();
		Iterator<E> iterThis = iterator();
		Object tmp = null;
		while(iter.hasNext())
		{
			tmp = iter.next();
			if(tmp == null)
			{
				if(iterThis.next() != null)
				{
					return false;
				}
			}
			else
			{
				if(!tmp.equals(iterThis.next())))
				{
					return false;
				}
			}
		}
		return true;
	}

	public boolean isEmpty()
	{
		return count == 0;
	}

	public boolean removeAll(Collection<?> c)
	{
		boolean retval = false;
		Iterator iter = c.iterator();
		while(iter.hasNext())
		{
			retval = retval || remove(iter.next());
		}
		return retval;
	}

	public boolean remove(Object o)
	{
		int i = 0;
		if(o == null)//cerco un null
		{
			for(i = 0;i < count;i++)
			{
				//se trovo esco dal loop
				if(storage[i] == null)
				{
					break;
				}
			}
		}
		else//uso equals per confrontare
		{
			for(i = 0;i < count;i++)
			{
				//se trovo esco dal loop
				if(o.equals(storage[i]))
				{
					break;
				}
			}
		}
		//se i vale meno di count, allora ho trovato l'elemento da eliminare
		//se i vale count, allora ho iterato su tutto il vettore senza successo
		if(i < count)
		{
			for(int k = i; k < count-1;k++)
			{
				storage[k] = storage[k+1];
			}
			storage[k] = null;
			count--;
			return true;
		}
		return false;
	}

	/*
	public boolean remove(Object o)
	{
		if(o == null)
		{
			for(int i = 0;i < count;i++)
			{
				if(storage[i] == null)
				{
					for(int k = i; k < count-1;k++)
					{
						storage[k] = storage[k+1];
					}
					storage[k] = null;
					count--;
					return true;
				}
			}
		}
		else
		{
			for(int i = 0;i < count;i++)
			{
				if(o.equals(storage[i]))
				{
					for(int k = i; k < count-1;k++)
					{
						storage[k] = storage[k+1];
					}
					storage[k] = null;
					count--;
					return true;
				}
			}
		}
		return false;
	}
	*/

	public boolean contains(Object o)
	{
		if(o == null)
		{
			for(int i = 0;i<count;i++)
			{
				if(storage[i] == null)
				{
					return true;
				}
			}
		}
		else
		{
			for(int i = 0;i<count;i++)
			{
				if(o.equals(storage[i]))
				{
					return true;
				}
			}
		}
		return false;
	}

	public boolean containsAll(Collection<?> c)
	{
		Iterator iter = c.iterator();
		while(iter.hasNext())
		{
			if(!this.contains(iter.next()))
			{
				return false;
			}
		}
		return true;
	}



	public Iterator<E> iterator()
	{
		return new ArrayCollectionIterator<E>(this);
	}

	class ArrayCollectionIterator<E>
	{
		boolean nextDone = false;
		int index;
		ArrayCollection ac;

		ArrayCollectionIterator(ArrayCollection theCollection)
		{
			index = 0;
			ac = theCollection;
		}

		public E next()
		{
			try
			{
				nextDone = true;
				return ac.storage[index++];
			}
			catch(ArrayIndexOutOfBoundsException aioobe)
			{
				NoSuchElementException nsee = new NoSuchElementException();
				nsee.initCause(aioobe);
				throw nsee;
			}
		}

		public boolean hasNext()
		{
			return index < ac.count;
		}

		public void remove()
		{
			if(!nextDone)
			{
				throw new IllegalStateException();
			}
			nextDone = false;
			if(index == ac.count)
			{
				ac.storage[--index] = null;
				ac.count--;
			}
			else
			{
				int i = 0;
				for(i = index;i<ac.count-1;i++)
				{
					ac.storage[i-1] = ac.storage[i];
				}
				ac.storage[i] = null;
				ac.count--;
				index--;
			}
		}
	}


}
